/*
*  unodio.h: for user programe use UNO DIO function
*
*  Copyright by  Advantech Co., Ltd. 2008 All rights reserved.
*/
#ifndef UNODIO_H
#define UNODIO_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/select.h>
#include <pthread.h>
#include <errno.h>

// Counter channel number
#define COUNTER_0       0x0  // 16-bit counter 0
#define COUNTER_1       0x1  // 16-bit counter 1
#define COUNTER_32      0x2  // Cascaded 32-bit counter of counter 0 and counter 1

// interrupt flag & clear (offset 7)
#define INT_DI0_MASK    0x1
#define INT_DI1_MASK    0x2
#define INT_CNTA_MASK   0x200
#define INT_CNTB_MASK   0x400

// Buzzer Mode
#define BEEP_DISABLE    0x0 // disable alarm buzzer
#define BEEP_ON         0x1 // beep on
#define BEEP_SHORT      0x3 // short beep
#define BEEP_MIDDLE     0x5 // normal beep
#define BEEP_LONG       0x7 // long beep
 
// LED Mode
#define LED_DISABLE     0x0  // disable diagnostic LED
#define LED_ON          0x1  // Light on
#define LED_FAST        0x3  // fast flicker
#define LED_NORMAL      0x5  // normal flick
#define LED_SLOW        0x7  // slow flicker

// Event Type
#define ADS_EVT_DI0     0x0001  // for DI 0
#define ADS_EVT_DI1     0x0002  // for DI 1 
#define ADS_EVT_DI2     0x0004  // for DI 2
#define ADS_EVT_DI3     0x0008  // for DI 3
#define ADS_EVT_DI4     0x0010  // for DI 4
#define ADS_EVT_DI5     0x0020  // for DI 5
#define ADS_EVT_DI6     0x0040  // for DI 6
#define ADS_EVT_DI7     0x0080  // for DI 7
#define ADS_EVT_DI8     0x0100  // for DI 8
#define ADS_EVT_CONT0   0x0200  // for counter 0
#define ADS_EVT_CONT1   0x0400  // for counter 1 or cascaded 32-bit counter

#define EVENT_EDGE_RISING  1  // interrpt trigger edge for rising edge
#define EVENT_EDGE_FALLING 0  // interrpt trigger edge for falling edge

// Internal clock's time base selecting (TimerBase)
#define TIME_BASE_100K  0x00    // time base 100K Hz
#define TIME_BASE_10K   0x10    // time base 10K Hz
#define TIME_BASE_1K    0x20    // time base 1K Hz
#define TIME_BASE_100   0x30    // time base 100 Hz

// clock source selecting (for ClockPin parameter)
#define CFG_CLOCK_INTERNAL  0   // clock source from internal clock
#define CFG_CLOCK_EXTERNAL  1   // clock source from external clock
                                // Counter 0 from DI1
                                // Counter 1 and cascaded 32-bit counter from DI3

// Gate tigger selecting (for GatePin paramete)
#define CFG_GATE_INTERNAL    0  // gate input from internal trigger
#define CFG_GATE_EXTERNAL    1  // gate input from external trigger
                                // Counter 0 from DI0
                                // Counter 1 and cascaded 32-bit counter from DI2

// Output pin selecting (for OutPin parameter)
#define CFG_OUTPIN_NON_DO    0  // Counter not output to DO channel
#define CFG_OUTPIN_TO_DO     1  // Counter output to DO channel
                                // Counter 0 output to DO2
                                // Counter 1 and cascaded 32-bit counter to DO3

// In usGATEPin=CFG_GATE_INTERNAL for CTRGATE parameter's definition
#define CFG_CTR_STOP    0       // counter pause
#define CFG_CTR_START   1       // counter start or resume
                                // Counter output - Output pin selecting (for OutPin parameter)
#define CFG_OUTPIN_NON_MONITOR  0  // Counter not output to CTR0Out or CTR1Out
#define CFG_OUTPIN_MONITOR      1  // Counter output pin to CTR0Out or CTR1Out


#define EVT_DI0     0x0001  // for DI 0
#define EVT_DI1     0x0002  // for DI 1 
#define EVT_DI2     0x0004  // for DI 2
#define EVT_DI3     0x0008  // for DI 3
#define EVT_DI4     0x0010  // for DI 4
#define EVT_DI5     0x0020  // for DI 5
#define EVT_DI6     0x0040  // for DI 6
#define EVT_DI7     0x0080  // for DI 7
#define EVT_DI8     0x0100 
#define EVT_CNT0    0x0200
#define EVT_CNT1    0x0400
#define EVT_TIMEOUT 0xFF 
#define IRQ_NOTIFICATION_CODE    0x10 

// -----------------------------------------------------------------------------
// DESCRIPTION: 
// -----------------------------------------------------------------------------
#define FUNC_DIO         0x01  //support DI,DO,DIINT
#define FUNC_COUNTER     0x02  //support COUNTER,TIMER,FREQ,PLUSE
#define FUNC_LED         0x04  //support LED
#define FUNC_PLED        0x08  //support PROGRAM LED
#define FUNC_BUZZER      0x10  //support buzzer
#define FUNC_POWERSTATUS 0x20  //support power status
#define FUNC_DIINT       0x40  //support DI interrupt

int DeviceOpen ( void);
int DeviceClose (void );
int GetPowerStatus ( int * usState );
int DIReadPortByte(int port, int *value);
int DOWritePortByte(int port, int mask, int value);
int DIReadBit(int port, int bit, int *state);
int DOWriteBit(int port, int bit, int state);
int DOGetCurrentByte(int port, int *value);
int DOGetCurrentBit(int port, int bit, int *state);
int CounterTimerStart(int counter, int initHWCount);
int CounterEventStart(int counter, int usCTRGate);
int CounterEventRead(int counter, int *overflow, int *count);
int CounterFreqStart(int counter,  int usCTRGate);
int CounterFreqRead(int counter, float *freq);
int CounterPulseStart(int counter, float period, int usCTRGate);
int CounterReset(int counter);
int CounterClockConfig(int timeBase);
int CounterConfig(int counter, int clockPin, int gatePin, int outPin);
int CounterGateSet(int counter, int CTRGate);
int CounterOutputMonitor(int counter, int *state);
int WritePortByte(int offset, int value);
int ReadPortByte(int offset, int* value);
int EventConfig(int eventType, int triggerEdge);
int EventEnable(int eventType, int enabled, int count);
int LedEnable(int mode);
int ProgramLedWrite(int mode,int mask);
int ProgramLedRead(int *value);
int ProgramLedModeEnable( int ledChannel, int Enabled )
;
int ProgramLedModeConfig( int usMode )
;
int BuzzerEnable(int mode);
int AddIntHandler(int event, void *handler);   
int CheckEvent(int timeout);
int GetDeviceInfor(int * DIChannelNum,int * DOChannelNum,int * bSupportFunc);

#endif

